const dotenv = require("dotenv");
dotenv.config();
let config = require("./endpoint");

const { Api, JsonRpc, RpcError } = require("eosjs");
const { JsSignatureProvider } = require("eosjs/dist/eosjs-jssig");
const signatureProvider = new JsSignatureProvider([]);
const dspEndpt = config.endpoint;
const fetch = require("isomorphic-fetch");
const rpc = new JsonRpc(dspEndpt, { fetch });
const { TextEncoder, TextDecoder } = require("text-encoding");

const api = new Api({
  rpc,
  signatureProvider,
  textDecoder: new TextDecoder(),
  textEncoder: new TextEncoder(),
});

const { createClient } = require("@liquidapps/dapp-client");
const Hashids = require("hashids/cjs");
const axios = require("axios").default;
const encoding = [
  "a",
  "b",
  "c",
  "d",
  "e",
  "f",
  "g",
  "h",
  "i",
  "j",
  "k",
  "l",
  "m",
  "n",
  "o",
];
Object.defineProperty(exports, "__esModule", { value: true });

exports.reguser = async (email, contact, otp, prvkey) => {
  try {
    const service = await getservice();
    const vacc = await this.converttovacc(email);
    let resp = await service.sign_liquid_account_transaction(
      config.contract,
      prvkey,
      "regboth",
      {
        vaccount: vacc,
        email: email,
        contact: contact,
        otp: otp,
        payload: {
          vaccount: vacc,
        },
      }
    );
    let pushresult = await pushTransaction(
      resp.public_key,
      resp.signature,
      resp.payload,
      []
    );
    if (pushresult) {
      return { accname: vacc };
    }
  } catch (err) {
    throw new Error(err);
  }
};

exports.recoveruser = async (email, otp, prvkey) => {
    try {
      const service = await getservice();
      const dappacc = await this.converttovacc(email);
      let resp = await service.sign_liquid_account_transaction(
        config.contract,
        prvkey,
        "confirmcode",
        {
          vaccount: dappacc,
          otp: otp,
          payload: {
            vaccount: dappacc,
          },
        }
      );
      let pushresult = await pushTransaction(
        resp.public_key,
        resp.signature,
        resp.payload,
        []
      );
      if (pushresult) {
        return { accname: dappacc };
      }
    } catch (err) {
      throw new Error(err);
    }
  };

async function gettrxbyid(trxid) {
  const dspEndpt = config.gettrx;
  const rpc = new JsonRpc(dspEndpt, { fetch });
  const api = new Api({
    rpc,
    signatureProvider,
    textDecoder: new TextDecoder(),
    textEncoder: new TextEncoder(),
  });
  let resp = await rpc.history_get_transaction(trxid);
  return resp;
}

async function getCorrectTransactionId(transactionIds) {
  const requests = transactionIds.map((v) => gettrxbyid(v));
  const responses = await axios.all(requests);
  let index = -1;

  for (let i = 0; i < responses.length; i++) {
    if (responses[i].error) {
      continue;
    } else {
      index = i;
      break;
    }
  }

  if (index >= 0) {
    return transactionIds[index];
  }

  return "";
}

async function pushTransaction(pubkey, signature, payload, txIds) {
  if (txIds.length) {
    const txId = await getCorrectTransactionId(txIds);
    if (txId.length > 0) {
      return true;
    }
  }

  try {
    const service = await getservice();
    const response = await service.post_liquid_account_transaction(
      config.contract,
      pubkey,
      signature,
      payload
    );
    const txId = response.result.transaction_id;
    txIds.push(txId);
    return pushTransaction(pubkey, signature, payload, txIds);
  } catch (e) {
    const error = JSON.stringify(e);
    if (
      error.includes("invalid nonce") ||
      error.includes("Provided public key already registered for this") ||
      error.includes("Call updateacckey to add keys")
    ) {
      return true;
    } else {
      throw new Error(e);
    }
  }
}
async function getservice() {
  let client = await createClient({
    network: "kylin",
    httpEndpoint: config.endpoint,
    fetch: fetch,
  });
  const service = await client.service("vaccounts", "vacctoken113");
  return service;
}

exports.converttovacc = async (email) => {
  const hashids = new Hashids(email, 9);
  let hashsh = hashids.encode(999);
  let accname = "dap" + (await getaccname(hashsh.toLowerCase()));
  return accname;
}

async function getaccname(data) {
  let newdata = "";
  for (let i = 0; i < data.length; i += 1) {
    if (data.charAt(i) === 0 || data.charAt(i) > 5 || data.charAt(i) === "0") {
      newdata += encoding[data.charAt(i)];
    } else newdata += data.charAt(i);
  }
  return newdata;
}
